# -*- coding: utf-8 -*-

"""Controller API for `clusters' part.

    Functionality description:

    Let's have the main subcontroller (ClusterController) available through
    'clusters' identifier of the root controller and subcontroller handling
    related commands (ClusterCmdController) through 'clusters_cmd' identifier.

    Luci application is accessible via http://example.com. Clusters
    in the system are ClusterOne, ClusterTwo and ClusterThree.

    Accessing following URLs has following consequences (some cases omitted):

    1) <http://example.com/clusters> OR <http://example.com/clusters/>
       - displaying a list of clusters with some information about them.

    2) <http://example.com/clusters/ClusterOne>
       OR <http://example.com/clusters/ClusterOne/>
       - redirection to nodes listing related to cluster called `ClusterOne'
         (<http://example.com/clusters/ClusterOne/nodes>) because there is no
         general page for certain cluster, only a few pages with further
         information about it (and the `nodes' is probably the most important).

    3) <http://example.com/clusters/NonExistingCluster>
       OR <http://example.com/clusters/NonExistingCluster/>
       OR <http://example.com/clusters/NonExistingCluster/something_more>
       - redirection back to base `clusters' page and displaying information
         about the request of non-existing cluster.

    ---

    4) <http://example.com/clusters_cmd/create>
       - display form for cluster creation.

    5) <http://example.com/clusters_cmd/add>
       - display form for existing cluster addition.

"""

from tg import flash, request, redirect, expose, tmpl_context, app_globals
from pylons.i18n import ugettext as _, lazy_ugettext as l_
from repoze.what import predicates

from luci.lib.base import SubController, SubControllerApplyMixin, \
                          ApplyCommands, setApplyCommands
from luci.controllers.decorators import *
from luci.lib.app_strings import Title, FlashMsg
from luci.lib.helpers import relativeUrlList2Str
from luci.widgets.create_cluster_form import create_cluster_form
from luci.widgets.add_existing_form import create_add_existing_form

__all__ = ['ClusterController', 'ClusterCmdController']


# Imports into module's namespace.
scheme = app_globals.scheme
cluster_scheme = app_globals.scheme.Cluster
data = app_globals.data
base2CmdCtrl = app_globals.scheme.base2CmdCtrl


#
# ITEMS LISTING.
#

class ClusterController(SubController):
    """Subcontroller handling basic requests related with `clusters' part."""

    # TODO: Uncomment this (or replace with more suitable predicate)
    # to apply access control.
    # For some reason this won't work for me, similar problem discussed at:
    # <http://groups.google.com.ar/group/turbogears/browse_thread/thread/28a8b855a850a1e7>
    # allow_only = predicates.not_anonymous()


    @forPageShowWithUrlCorrection('luci.templates.cluster')
    def default(self):
        """Handle simple clusters listing.

        E.g. <http://example.com/clusters> for situation described
        in the module's doc.

        """
        Title.usePart('CLUSTERS')

        base_url_str = relativeUrlList2Str(scheme.APP_PREFIX, scheme.CLUSTERS)
        cmd_url_str = relativeUrlList2Str(scheme.APP_PREFIX,
                                          base2CmdCtrl(scheme.CLUSTERS))

        return dict(base_url=base_url_str, page='clusters',
                    cmd_url=cmd_url_str,
                    apply_cmds=ClusterApplyCommands)


    @expose()
    def lookup(self, name, *args):
        """Dynamic dispatching to subcontrollers according to cluster's name.

        E.g. <http://example.com/clusters/ClusterOne> for situation described
        in the module's doc.

        The request is dynamically delegated to _CertainClusterController
        object.

        Keyword arguments:
            name    Name of the cluster to be taken as a context for
                    nested subcontrollers (NodeController, etc.).

        """
        if name in data.clusters.iterkeys():
            # Following 3 lines necessary because of strange(?) behavior of TG
            # -- usually 3 iteration over controller's stack.
            if not tmpl_context.already_used.get('cluster'):
                Title.usePart('CERTAIN_CLUSTER', name)
                tmpl_context.cluster_name = name
                tmpl_context.cluster_data = data.clusters[name]
                tmpl_context.cluster_url = \
                    relativeUrlList2Str(scheme.APP_PREFIX, scheme.CLUSTERS, name)
                tmpl_context.already_used['cluster'] = True
            dynamic_ctrl = _CertainClusterController()
            return dynamic_ctrl, args
        else:
            msg = l_('Bad cluster name: %s') % name
            status = 'error'

        flash(msg, status=status)
        redirect(relativeUrlList2Str(scheme.APP_PREFIX, scheme.CLUSTERS))


@mountSubControllers(cluster_scheme.mounts)
class _CertainClusterController(SubController):
    """Subcontroller preparing subcontrollers for certain cluster."""

    @forImmediateRedirect(allowed_methods='GET')
    def default(self, *args, **kwargs):
        """Handle request for displaying details of certain cluster.

        E.g. <http://example.com/clusters/ClusterOne> for situation described
        in the module's doc.

        Redirect to `nodes' part in the context the cluster.

        """
        return dict(use_referrer=False,
                    redir_target=relativeUrlList2Str(tmpl_context.cluster_url,
                                                     cluster_scheme.NODES))

    # 'lookup' method not necessary, 'default' method above is enough even for
    # such cases.


#
# COMMANDS HANDLING.
#

class ClusterApplyCommands(ApplyCommands):
    """Encapsulation of available commands to apply over selected cluster(s)."""

    DELETE = 'cmd_delete'


    def _delete(which):
        clusters = app_globals.data.clusters
        msg = l_('Deleting selected cluster(s)... %s') \
                  % (', '.join(which))
        status = 'info'
        for cluster in which:
            if clusters.has_key(cluster):
                del clusters[cluster]
            else:
                # If Luci's data are consistent, this should never happen.
                msg, status = FlashMsg.INTERNAL_ERROR
                break
        return msg, status, False


    # Mapping.
    cmds = {DELETE: _delete}

    # Set methods as static (but only after they are referenced
    # in 'cmds' dict).
    _delete = staticmethod(_delete)


@setApplyCommands(ClusterApplyCommands)
class ClusterCmdController(SubController, SubControllerApplyMixin):
    """Subcontroller handling commands related with `clusters' part."""

    # TODO: Uncomment this (or replace with more suitable predicate)
    # to apply access control.
    # For some reason this won't work for me, similar problem discussed at:
    # <http://groups.google.com.ar/group/turbogears/browse_thread/thread/28a8b855a850a1e7>
    # allow_only = predicates.not_anonymous()


    @forPageShowWithUrlCorrection('luci.templates.cluster_forms')
    def create(self, name=''):
        """Display form for cluster creation."""

        Title.usePart('CLUSTER_CREATE')

        tmpl_context.form = create_cluster_form

        base_url_str = relativeUrlList2Str(scheme.APP_PREFIX, scheme.CLUSTERS)
        cmd_url_str = relativeUrlList2Str(scheme.APP_PREFIX,
                                          base2CmdCtrl(scheme.CLUSTERS))

        return dict(base_url=base_url_str,
                    cmd_url=cmd_url_str,
                    page='create_cluster')


    @forPageShowWithUrlCorrection('luci.templates.cluster_forms')
    def add_existing(self):
        """Display form for existing cluster addition."""

        Title.usePart('CLUSTER_ADD_EXISTING')

        tmpl_context.form = create_add_existing_form

        base_url_str = relativeUrlList2Str(scheme.APP_PREFIX, scheme.CLUSTERS)
        cmd_url_str = relativeUrlList2Str(scheme.APP_PREFIX,
                                          base2CmdCtrl(scheme.CLUSTERS))

        return dict(base_url=base_url_str,
                    cmd_url=cmd_url_str,
                    page='add_existing')

    @forImmediateRedirect(allowed_methods='POST')
    def cluster_create(self, **kw):
        from luci.widget_validators.validate_create_cluster_form import validate_create_cluster_form
        return validate_create_cluster_form(self, **kw)
