# -*- coding: utf-8 -*-

"""Controller API for `nodes of certain cluster' part.

    Functionality description:

    Let's have the main subcontroller (NodeController) available through
    'nodes' identifier of the `clusters' controller and the subcontroller
    handling related commands (NodeCmdController) through 'nodes_cmd'
    identifier.

    Main page of certain cluster (`ClusterOne') is accessible via
    http://example.com/clusters/ClusterOne. Nodes in the cluster are
    NodeAlpha, NodeBeta and NodeGamma.

    Accessing following URLs has following consequences (some cases omitted):

    1) <http://example.com/clusters/ClusterOne/nodes>
       OR <http://example.com/clusters/ClusterOne/nodes/>
       - displaying a list of nodes with some information about them.

    2) <http://example.com/clusters/ClusterOne/nodes/NodeAlpha>
       OR <http://example.com/clusters/ClusterOne/nodes/NodeAlpha/>
       - the same list as in case 1) is displayed, but also details to node
         `NodeAlpha' are shown in the lower part of the page.

    3) <http://example.com/clusters/ClusterOne/nodes/NonExistingNode>
       OR <http://example.com/clusters/ClusterOne/nodes/NonExistingNode/>
       OR <http://example.com/clusters/ClusterOne/nodes/NonExistingNode/something_more>
       - redirection back to base `nodes' page and displaying information
         about the request of non-existing node.

    ---

    4) <http://example.com/clusters/ClusterOne/nodes_cmd/add>
       - display form for a node addition to cluster.

    5) <http://example.com/clusters/ClusterOne/nodes_cmd/apply>
       OR <http://example.com/clusters/ClusterOne/nodes_cmd/apply?CMDID...>
       - apply a command on given node(s); both GET and POST method supported.

"""
from tg import flash, redirect, expose, tmpl_context, app_globals
from pylons.i18n import ugettext as _, lazy_ugettext as l_
from repoze.what import predicates

from luci.lib.base import SubController, SubControllerApplyMixin, \
                          ApplyCommands, setApplyCommands
from luci.controllers.decorators import *
from luci.lib.app_strings import Title, FlashMsg
from luci.lib.helpers import relativeUrlList2Str

__all__ = ['NodeController', 'NodeCmdController']


# Imports into module's namespace.
form_utils = app_globals.form_utils
cluster_scheme = app_globals.scheme.Cluster
base2CmdCtrl = app_globals.scheme.base2CmdCtrl


#
# ITEMS LISTING.
#

class NodeController(SubController):
    """Subcontroller handling basic requests related with `nodes' part."""

    # TODO: Uncomment this (or replace with more suitable predicate)
    # to apply access control.
    # For some reason this won't work for me, similar problem discussed at:
    # <http://groups.google.com.ar/group/turbogears/browse_thread/thread/28a8b855a850a1e7>
    # allow_only = predicates.not_anonymous()


    @forPageShowWithUrlCorrection('luci.templates.cluster_part.node')
    def default(self):
        """Handle simple nodes listing.

        E.g. <http://example.com/clusters/ClusterOne/nodes> for situation
        described in the module's doc.

        """
        Title.usePart('NODES')

        cluster_url = tmpl_context.cluster_url + u'/'

        return dict(page='nodes',name=None,
                    base_url=cluster_url + cluster_scheme.NODES,
                    cmd_url= \
                        cluster_url + base2CmdCtrl(cluster_scheme.NODES),
                    service_cmd_url= \
                        cluster_url + base2CmdCtrl(cluster_scheme.SERVICES),
                    apply_cmds=NodeApplyCommands)


    @expose()
    def lookup(self, name, *args):
        """Handle nodes listing with details of certain one.

        E.g. <http://example.com/clusters/ClusterOne/nodes/NodeAlpha>
        for situation described in the module's doc.

        The request is dynamically delegated to _CertainNodeController object.

        Keyword arguments:
            name    Name of the node to be displayed in more detail.

        """
        # Check whether required node exists.
        if tmpl_context.cluster_data.nodes.has_key(name):
            dynamic_ctrl = _CertainNodeController(entity_name=name)
            return dynamic_ctrl, args
        else:
            msg = l_('Bad node name: %s') % name
            status = 'error'

        flash(msg, status=status)
        redirect(relativeUrlList2Str(tmpl_context.cluster_url, cluster_scheme.NODES))


class _CertainNodeController(SubController):
    """Subcontroller handling nodes listing with details of certain one."""

    @forPageShowWithUrlCorrection('luci.templates.cluster_part.node')
    def default(self):
        """Handle nodes listing with details of certain one.

        E.g. <http://example.com/clusters/ClusterOne/nodes/NodeAlpha>
        for situation described in the module's doc.

        """
        Title.usePart('CERTAIN_NODE', self.entity_name)

        cluster_url = tmpl_context.cluster_url + u'/'

        return dict(page='nodes', name=self.entity_name,
                    base_url=cluster_url + cluster_scheme.NODES,
                    cmd_url= \
                        cluster_url + base2CmdCtrl(cluster_scheme.NODES),
                    service_cmd_url= \
                        cluster_url + base2CmdCtrl(cluster_scheme.SERVICES),
                    apply_cmds=NodeApplyCommands)


    @expose()
    def lookup(self, *args):
        """Handle wrong URL (longer than expected) connected with certain node.

        E.g. <http://example.com/clusters/ClusterOne/nodes/NodeAlpha/something>
        for situation described in the module's doc.

        """
        redirect(relativeUrlList2Str(tmpl_context.cluster_url, cluster_scheme.NODES,
                                self.entity_name))


#
# COMMANDS HANDLING.
#

class NodeApplyCommands(ApplyCommands):
    """Encapsulation of available commands to apply over selected node(s)."""

    REBOOT = 'cmd_reboot'
    FENCE = 'cmd_fence'
    LEAVE = 'cmd_leave'
    DELETE = 'cmd_delete'


    def _reboot(which):
        msg = l_('Rebooting selected node(s)... %s') \
              % (", ".join(which))
        status = 'info'
        return msg, status


    def _fence(which):
        msg = l_('Fencing selected node(s)... %s') \
              % (", ".join(which))
        status = 'info'
        return msg, status


    def _leave(which):
        msg = l_('Removing selected node(s) from the cluster... %s') \
                % (", ".join(which))
        status = 'info'
        return msg, status


    def _delete(which):
        nodes = tmpl_context.cluster_data.nodes
        msg = l_('Deleting selected node(s)... %s') \
                  % (', '.join(which))
        status = 'info'
        for node in which:
            if nodes.has_key(node):
                del nodes[node]
            else:
                # If Luci's data are consistent, this should never happen.
                msg, status = FlashMsg.INTERNAL_ERROR
                break
        return msg, status, False


    # Mapping.
    cmds = {REBOOT: _reboot,
            FENCE: _fence,
            LEAVE: _leave,
            DELETE: _delete}

    # Set methods as static (but only after they are referenced
    # in 'cmds' dict).
    _reboot = staticmethod(_reboot)
    _fence = staticmethod(_fence)
    _leave = staticmethod(_leave)
    _delete = staticmethod(_delete)


@setApplyCommands(NodeApplyCommands)
class NodeCmdController(SubController, SubControllerApplyMixin):
    """Subcontroller handling commands related with `nodes' part."""

    # TODO: Uncomment this (or replace with more suitable predicate)
    # to apply access control.
    # For some reason this won't work for me, similar problem discussed at:
    # <http://groups.google.com.ar/group/turbogears/browse_thread/thread/28a8b855a850a1e7>
    # allow_only = predicates.not_anonymous()


    @forImmediateRedirect(allowed_methods = ('GET', 'POST'))
    def default(self, *args, **kwargs):
        """Handle requests not connected with any of defined method."""

        flash(*FlashMsg.BAD_COMMAND_REQUEST)

        base_url_str = relativeUrlList2Str(tmpl_context.cluster_url, self.base_name)
        return dict(redir_target=base_url_str)


    @forImmediateRedirect(allowed_methods = 'GET')
    def add(self):
        """Handle creation of a new node."""

        flash('It should be a dialog to add new node here instead, '
              'but not implemented yet...', status='info')
        # following code is only my experiment, how to continue:
        #tmpl_context.form = create_fdom_add_form
        #return dict()


    @forImmediateRedirect(allowed_methods = 'GET')
    def update_settings(self, name=None):
        """Handle updating of settings."""

        flash('Demo of updating settings for node %s...' % name,
              status='info')

        base_url_str = relativeUrlList2Str(tmpl_context.cluster_url, self.base_name)
        return dict(redir_target=base_url_str)


    @forImmediateRedirect(allowed_methods = 'GET')
    def update_properties(self, name=None):
        """Handle updating of properties."""

        flash('Demo of updating properties for node %s...' % name,
              status='info')

        base_url_str = relativeUrlList2Str(tmpl_context.cluster_url, self.base_name)
        return dict(redir_target=base_url_str)

