import os
from datetime import datetime
import sys

from sqlalchemy import Table, ForeignKey, Column
from sqlalchemy.types import Unicode, Integer, DateTime, String
from sqlalchemy.orm import relation, synonym

from luci.model import DeclarativeBase, metadata, DBSession

cluster_nodes_table = Table('cluster_nodes', metadata,
    Column('cluster_id', Integer, ForeignKey('clusters.cluster_id'), primary_key=True),
    Column('node_id', Integer, ForeignKey('nodes.node_id'), primary_key=True))

cluster_tasks_table = Table('cluster_tasks', metadata,
    Column('cluster_id', Integer, ForeignKey('clusters.cluster_id'), primary_key=True),
    Column('task_id', Integer, ForeignKey('tasks.task_id'), primary_key=True))

node_tasks_table = Table('node_tasks', metadata,
    Column('node_id', Integer, ForeignKey('nodes.node_id'), primary_key=True),
    Column('task_id', Integer, ForeignKey('tasks.task_id'), primary_key=True))

class Task(DeclarativeBase):
    __tablename__ = 'tasks'

    task_id = Column(Integer, autoincrement=True, primary_key=True)
    batch_id = Column(Integer)
    batch_status = Column(Integer)
    started = Column(DateTime, default=datetime.now)
    last_status = Column(DateTime, default=datetime.now)
    finished = Column(DateTime)
    description = Column(Unicode(255))
    status_msg = Column(Unicode(255))
    redirect_url = Column(Unicode(512))

    def __repr__(self):
        return '<Task: id=%d batchid=%d>' % (self.task_id, self.batch_id)
    
    def __unicode__(self):
        return self.task_id


class Node(DeclarativeBase):
    __tablename__ = 'nodes'
    
    #{ Columns
    
    node_id = Column(Integer, autoincrement=True, primary_key=True)
    node_name = Column(Unicode(16), unique=True, nullable=False)
    display_name = Column(Unicode(255))
    hostname = Column(Unicode(255))
    ipaddr = Column(String(16))
    port = Column(Integer)
    
    #{ Relations
    tasks = relation(Task, secondary=node_tasks_table, backref="node")
    
    #{ Special methods
    
    def __repr__(self):
        return '<Node: name=%s>' % self.node_name
    
    def __unicode__(self):
        return self.node_name
    
    #}

class Cluster(DeclarativeBase):
    __tablename__ = 'clusters'
    
    #{ Columns

    cluster_id = Column(Integer, autoincrement=True, primary_key=True)
    name = Column(Unicode(16), unique=True, nullable=False)
    display_name = Column(Unicode(255))
    created = Column(DateTime, default=datetime.now)

    #{ Relations

    nodes = relation(Node, secondary=cluster_nodes_table, backref="cluster")
    tasks = relation(Task, secondary=cluster_tasks_table, backref="cluster")
    
    #{ Special methods

    def __repr__(self):
        return '<Cluster: id="%d" name="%s" display_name="%s">' % (
                self.cluster_id, self.name, self.display_name)

    def __unicode__(self):
        return self.display_name or self.name
